#!/usr/bin/python

import argparse
import os
import subprocess
import sys
import yaml  # need package PyYaml...
import logging

from neutronclient.v2_0 import client
from neutronclient.neutron.v2_0 import port as _port

from modev import openstack
from modev import util

DEFAULT_CONF_DIR = os.path.expanduser('~/.modev/')
DEFAULT_CONFIG = DEFAULT_CONF_DIR + "config.yaml"
FIXED_PREFIX = "modev_"
DEFAULT_REGION = "m0d"
DEFAULT_EXTERNAL_NET = None
DEFAULT_ADM_SUBNET = "172.31.200.0/24"
DEFAULT_CLUSTER_SUBNET = "172.31.201.0/24"
LOG_FILE = "/tmp/modev.log"

CFG_NAMES = (
    'os_auth_url',
    'os_insecure',
    'os_password',
    'os_region_name',
    'os_tenant_name',
    'os_tenant_id',
    'os_username',
)

# to do something better with dpkg -s
DEPENDANCIES = {
    'cryptsetup': 'cryptsetup',
    'openssh-client': 'ssh',
    'python-novaclient': 'nova',
    'python-neutronclient': 'neutron',
    'python-glanceclient': 'glance',
    'python-swiftclient': 'swift',
    'qemu-utils': 'qemu-img',
}

COMMON_ARGS = []

# COMMON_FLAGS = {
#    'config': (('--config',),
#               {'help': 'modev config to use', 'default': DEFAULT_CONFIG}),
#    'dry-run': (('-n', '--dry-run'),
#                {'help': 'only report what would be done',
#                 'action': 'store_true', 'default': False}),
#}

SUBCOMMANDS = {
    'check': {
        'help': 'check if we have all needed package installed',
        'opts': [
            (('--yes', '-y'),
             {'help': 'don\'t ask any questions, install if not there',
                 'action': 'store_true', 'default': False}),
            (('--update', '-u'),
             {'help': 'update before checking packages',
                 'action': 'store_true', 'default': False}),
        ]
    },
    'region-create': {
        'help': 'create a config with defaults',
        'opts': [
            #           COMMON_FLAGS['config'],

            (('--net-extern', '-nex'),
             {'help': 'the external openstack network',
              'action': 'store', 'default': DEFAULT_EXTERNAL_NET}),

            #            (('--maas-creds',),
            #             {'help': 'maas credential to use',
            #              'action': 'store', 'default': None}),

            #            (('--maas-url',),
            #             {'help': 'maas url to use',
            #              'action': 'store', 'default': None}),

            (('--key-priv', '-kprv'),
             {'help': 'ssh private key to use (don\'t create one)'}),

            (('--key-pub', '-kpub'),
             {'help': 'ssh public key to use (don\'t create one)'}),

            (('--key-pem', '-kpem'),
             {'help': 'pem file to use (don\'t create one)'}),

            (('--net-router', '-nr'),
             {'help': 'if you want to reuse an existing router'}),

            (('--force', '-f'),
             {'help': 'if leftover config files exists',
              'action': 'store_true', 'default': False}),

            (('openrc',),
             {'help': 'the openstack rc to use', 'action': 'store'}),

            (('--region-name', '-rn'),
             {'help': 'the prefix for openstack items',
              'action': 'store', 'default': DEFAULT_REGION}),
        ]
    },
    'region-purge': {
        'help': 'purge everything in a region',
        'opts': [
            #            COMMON_FLAGS['config'],

            (('--all-regions',),
             {'help': 'purge all regions and all config files',
              'action': 'store_true'}),

            (('--same-cloud', '-sc'),
             {'help': 'purge any region found using the openrc env' +
              ' of the region name specified by -rn',
              'action': 'store_true', 'default': False}),

            (('--openrc', '-rc'),
             {'help': 'purge region(s) with just the openrc ' +
                      'if no region name is given, all are destroyed'}),

            (('--region-name', '-rn'),
             {'help': 'the region to delete must be indicated',
              'action': 'store'})
        ]
    },
    'net-extern-list': {
        'help': 'list all net available for a region or openrc',
        'opts': [
            #            COMMON_FLAGS['config'],

            (('--openrc', '-rc'),
             {'help': 'openrc to list external network from'}),

            (('--region-name', '-rn'),
             {'help': 'get the list from a region config',
              'action': 'store'})
        ]
    },
    'region-list': {
        'help': 'list all regions, *: current active region',
        'opts': [
            #            COMMON_FLAGS['config'],

            (('--openrc', '-rc'),
             {'help': 'openrc to list region from'}),

            (('--region-name', '-rn'),
             {'help': 'list region on the same cloud than this one'}),
        ]
    },
    'region-switch': {
        'help': 'switch the default region to use',
        'opts': [
            #            COMMON_FLAGS['config'],
            (('region-name',),
             {'help': 'the region name to switch to',
              'action': 'store'}),
        ]
    },
    'region-current': {
        'help': 'display current active region',
        'opts': [
        ]
    },
    'td': {
        'help': 'to be discarded, quick access to functions',
        'opts': [
            (('--rn',),
             {'help': '', })
        ]
    },
}

'''
    'net-base-conf': {
        'help': 'create neutron base neutron networks',
        'opts': [
            COMMON_FLAGS['config'],
            (('--net-base-subnet',),
             {'help': 'you can define your own admin subnet \
                (default {})'.format(DEFAULT_ADM_SUBNET),
              'action': 'store', 'default': 'DEFAULT_ADM_SUBNET'}),

            (('--net-extern-allow',),
             {'help': 'allow external network access (default True)',
              'action': 'store', 'default': True})
        ]
    },
    'net-base-state': {
        'help': 'Return the state of the neutron base-setup',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'net-extern': {
        'help': 'allow to change from external network, \
            be carefull about floating IPs',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'cluster-add': {
        'help': 'add a new cluster network in a region',
        'opts': [
            COMMON_FLAGS['config'],
            (('--net-base-subnet'),
             {'help': 'subnet definition \
                (default {})'.format(DEFAULT_CLUSTER_SUBNET),
              'action': 'store', 'default': 'DEFAULT_CLUSTER_SUBNET'}),

            (('name'),
             {'help': 'name of the cluster', 'action': 'store',
              'default': 'cluster-01'}),

            (('--cluster-controller-create'),
             {'help': 'create the maas cluster controller instance \
                on this cluster',
              'action': 'store', 'default': True}),

            (('name'),
             {'help': 'name of the cluster', 'action': 'store',
              'default': 'cluster-01'}),
        ]
    },
    'cluster-delete': {
        'help': 'delete a cluster network from a region',
        'opts': [
            COMMON_FLAGS['config'],
            (('name'),
             {'help': 'name of the cluster', 'action': 'store',
              'default': 'cluster-01'}),

            (('--nodes-purge'),
             {'help': 'delete all the nodes', 'action': 'store',
              'default': False}),

            (('--nodes-reallocate'),
             {'help': 'reallocate all the nodes to an other cluster',
              'action': 'store', 'default': 'cluster_def'}),

            (('name'),
             {'help': 'name of the cluster', 'action': 'store',
              'default': 'cluster-01'}),
        ]
    },
    'cluster-list': {
        'help': 'List all clusters in this region',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'cluster-node-list': {
        'help': 'List all node in a cluster',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'cluster-node-add': {
        'help': 'add node(s) to a specified cluster',
        'opts': [
            COMMON_FLAGS['config'],
            (('--number'),
             {'help': 'number of node to create', 'action': 'store',
              'default': 1}),

            (('--profile'),
             {'help': 'name of the profile to use', 'action': 'store',
              'default': 'default'}),

            (('cluster-name'),
             {'help': 'name of the cluster to attach to',
              'action': 'store', 'default': 'cluster_def'}),

            (('name'),
             {'help': 'base name of the node to create',
              'action': 'store', 'default': 'cluster-01'}),
        ]
    },
    'cluster-node-delete': {
        'help': 'delete node in a specified cluster',
        'opts': [
            COMMON_FLAGS['dry-run'],
            COMMON_FLAGS['config'],
            (('cluster-name'),
             {'help': 'name of the cluster this node belongs to',
              'action': 'store', 'default': None}),

            (('name'),
             {'help': 'base name of the node to create',
              'action': 'store', 'default': None}),
        ]
    },
    'node-list': {
        'help': 'list all nodes from this region',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'node-add': {
        'help': 'simply add an admin node',
        'opts': [
            COMMON_FLAGS['config'],
            (('--number'),
             {'help': 'number of node to create', 'action': 'store',
              'default': 1}),

            (('--cluster-name'),
             {'help': 'name of the cluster to attach to',
              'action': 'store', 'default': 'cluster_admin'}),

            (('--profile'),
             {'help': 'name of the profile to use', 'action': 'store',
              'default': None}),

            (('name'),
             {'help': 'name of the node to create', 'action': 'store',
              'default': None}),
        ]
    },
    'node-delete': {
        'help': 'delete an admin node',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'base name of the node to create',
              'action': 'store', 'default': None}),
        ]
    },
    'node-console': {
        'help': 'console log of the node',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'base name of the node to create',
              'action': 'store', 'default': None}),

            (('--cluster-name'),
             {'help': 'cluster name of the node', 'action': 'store',
              'default': None}),
        ]
    },
    'node-cluster-change': {
        'help': 'move this node to another cluster',
        'opts': [
            COMMON_FLAGS['config'],
            (('--cluster-name'),
             {'help': 'cluster name to move the node on',
              'action': 'store', 'default': None}),
        ]
    },
    'node-profile-list': {
        'help': 'list node profiles',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'node-profile-add': {
        'help': 'Add a new node profile',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'name of the new profile', 'action': 'store',
              'default': None}),

            (('--flavor-name'),
             {'help': 'name of the flavor to use', 'action': 'store',
              'default': None}),

            (('--add-disks'),
             {'help': 'number of ephemeral disks to add',
              'action': 'store', 'default': None}),

            (('--disk-size'),
             {'help': 'size of added ephemeral disks (GiB)',
              'action': 'store', 'default': 8}),

            (('--role-name'),
             {'help': 'if you need to forward user-data',
              'action': 'store', 'default': None}),
        ]
    },
    'node-profile-delete': {
        'help': 'delete an existing profile...',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'name of the new profile', 'action': 'store',
              'default': None}),
        ]
    },
    'node-role-list': {
        'help': 'List roles defined',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'node-role-add': {
        'help': 'add a new role',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'name of the new role', 'action': 'store',
              'default': None}),

            (('--user-data'),
             {'help': 'user-data to send to instance', 'action': 'store',
              'default': None}),
        ]
    },
    'node-role-del': {
        'help': 'delete a role',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'name of the role to delete', 'action': 'store',
              'default': None}),
        ]
    },
    'maas-all-in-one': {
        'help': 'Easy creation of a maas instance on an openstack cloud',
        'opts': [
            #            (('region-name',),
            #            {'help': 'specify a new region-name',
            #             'action': 'store',
            #             'default': FIXED_PREFIX + DEFAULT_REGION })
        ]
    },
    'maas-with-clusters': {
        'help': 'Easy creation of a maas instance with separate \
            clusters on an openstack cloud \n',
        'opts': [
            (('--cluster-add-number',),
             {'help': 'add any number of cluster (default: 2)',
              'action': 'store', 'default': 2}),

            (('--node-add-number',),
             {'help': 'add any number of nodes per cluster (default: 3)',
              'action': 'store', 'default': 3}),

            (('--cluster-controller-create',),
             {'help': 'create the maas cluster controller instance on \
                this cluster (default: True)',
              'action': 'store', 'default': True}),
            #            (('region-name',),
            #            {'help': 'specify a new region-name',
            #             'action': 'store',
            #             'default': FIXED_PREFIX + DEFAULT_REGION })
        ]
    },
    'maas-region-controller-list': {
        'help': 'list the maas region controller instances',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'maas-region-controller-add': {
        'help': 'add a new mass region controller (first one or HA)',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'maas-region-controller-del': {
        'help': 'delete a maas region controller',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'name of the maas region controller to delete',
              'action': 'store', 'default': None}),
        ]
    },
    'maas-cluster-controller-list': {
        'help': 'list all maas cluster controller of the region',
        'opts': [
            COMMON_FLAGS['config'],
        ]
    },
    'maas-cluster-controller-add': {
        'help': 'add a new maas cluster controller',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'name of the maas cluster controller to be added',
              'action': 'store', 'default': None}),

            (('--cluster-name'),
             {'help': 'name of the cluster to attach to',
              'action': 'store', 'default': 'cluster_def'}),
        ]
    },
    'maas-cluster-controller-del': {
        'help': 'del a maas cluster controller',
        'opts': [
            COMMON_FLAGS['config'],
            (('--name'),
             {'help': 'name of the maas cluster controller to be added',
              'action': 'store', 'default': None}),

            (('--cluster-name'),
             {'help': 'name of the cluster to remove from',
              'action': 'store', 'default': 'cluster_def'}),
        ]
    },
    'maas-get-creds': {
        'help': ('get the provided users credentials from local '
                 ' maas region controller [requires root]'),
        'opts': [
            (('user',),
             {'help': 'name of user', 'default': None, 'nargs': 1}),
        ],
    },
    'maas-create-user': {
        'help': ('create an admin user on local maas [requires root]'),
        'opts': [
            (('user',),
             {'help': 'name of user', 'default': None, 'nargs': 1}),

            (('password',),
             {'help': 'password', 'default': None, 'nargs': '*'}),
        ],
    'region-enter': {
        'help': 'enter a previously generated region',
        'opts': [
            (('region-name',),
             {'help': 'the prefix for openstack items',
              'action': 'store', 'default': DEFAULT_REGION}),
        ]
    },
    'region-backup': {
        'help': 'backup the current config',
        'opts': [
            (('--no-secret',),
             {'help': 'don\'t crypt the file (default: False)',
              'action': 'store'}),

            (('file',),
             {'help': 'the file to backup to',
              'action': 'store'}),
        ]
    },
    'region-restore': {
        'help': 'restore the config for current region',
        'opts': [
            (('file',),
             {'help': 'the file to restore from',
              'action': 'store'}),
        ]
    },
    'region-import': {
        'help': 'import a config from an existing modev instance',
        'opts': [
            (('openrc',),
             {'help': 'the openstack rc to retrieve config from',
              'action': 'store'}),
        ]
    },
    'region-export': {
        'help': 'export current config over ssh',
        'opts': [
            (('--user',),
             {'help': 'user on remote server', 'action': 'store'}),

            (('destination',),
             {'help': 'server or file to export file to',
              'action': 'store'}),
        ]
    },
'''


def rmdir(d=None):
    if os.path.exists(d):
        print('deleting directory:"{}"'.format(d))
        for path in (os.path.join(d, f) for f in os.listdir(d)):
            if os.path.isdir(path):
                rmdir(path)
            else:
                os.remove(path)
        os.rmdir(d)


def which(program):
# Following function borrowed from
# http://bazaar.launchpad.net/~cloud-init-dev/cloud-init/trunk/
# view/794/cloudinit/util.py#L1840
    # Return path of program for execution if found in path
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    _fpath, _ = os.path.split(program)
    if _fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ.get("PATH", "").split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None


def load_openrc(**kwargs):

    env_creds = openstack.load_keystone_creds()

    cfg = {}

    if env_creds:
        openstack.validate_keystone_creds(**env_creds)
        cfg.update({"os_" + k: v for k, v in env_creds.items()})

    if kwargs.get('openrc'):
        openrc_file = kwargs['openrc']
        if not os.path.isfile(openrc_file):
            print('"{}" this openrc file doesn\'t exist!'.format(openrc_file))
            sys.exit()

        shcfg = util.parse_shell_config(filepath=openrc_file)
        cfg.update({k.lower(): v for k, v in shcfg.items()
                    if k.startswith("OS_")})
    return cfg


def load_region_file(name=None):
    if name is None:
        raise NameError('Region Name is None!!!!')
    region_file = DEFAULT_CONF_DIR + name + '/' + name + '.conf'
    if not os.path.exists(region_file):
        print('This region doesn\'t seems to be configured:' +
              ' "{}"'.format(name))
        sys.exit()

    region_file = open(region_file, 'r')
    region_conf = yaml.safe_load(region_file)
    region_file.close()

#    return region_conf[FIXED_PREFIX + name]
    return region_conf


def get_region_rc(name=None):
    return(load_region_file(name=name)['openrc'])


def get_region_config(name=None):
    return(load_region_file(name=name)['config'])


def get_region_conf_rc(name=None):
    conf = load_region_file(name=name)
    return(conf['config'], conf['openrc'])


def list_modev_regions():
    region_list = []
    for path in os.listdir(DEFAULT_CONF_DIR):
        if is_modev_region(path):
            region_list.append(path)
    return(region_list)


def is_modev_region(region=None):
    if os.path.exists(DEFAULT_CONF_DIR + '/' + region + '/' + region +
                      '.conf'):
        return(True)
    else:
        return(False)


def get_modev_region():
    modev_region = None
    if os.path.exists(DEFAULT_CONFIG):
        modev_file = open(DEFAULT_CONFIG, 'r')
        modev_cfg = yaml.safe_load(modev_file)
        modev_region = modev_cfg['options']['region']['default']
        modev_file.close()
    else:
        print('Warning, the default config file is not found!' +
              '{}'.format(DEFAULT_CONFIG))
    return(modev_region)


def set_modev_region(region=None):
    if region is None:
        region = DEFAULT_REGION
    if os.path.exists(DEFAULT_CONFIG):
        modev_file = open(DEFAULT_CONFIG, 'r')
        modev_cfg = yaml.safe_load(modev_file)
        modev_file.close()
        modev_file = open(DEFAULT_CONFIG, 'w')
        modev_cfg['options']['region']['default'] = region
        modev_file.write(yaml.dump(modev_cfg))
        modev_file.close()


def select_region(region=None, config=None):
    modev_region = get_modev_region()
    if region is None:
        if modev_region is None:
            region = DEFAULT_REGION
        else:
            region = modev_region
    else:
        if '_' in region:
            print('You can\'t put a "_" in a region name')
            sys.exit()
    return region


def neutron_connect(openrc_cfg=None):
    if openrc_cfg is None:
        raise NameError('neutron_connect() called with wrong parameter')
#    print('Neutron_Connect_OpenRC:\n{}\n'.format(openrc_cfg))
    neutron = client.Client(username=openrc_cfg['os_username'],
                            password=openrc_cfg['os_password'],
                            tenant_name=openrc_cfg['os_tenant_name'],
                            auth_url=openrc_cfg['os_auth_url'])
    neutron.format = 'json'
    return neutron


def neutron_get_objects(name=None, objects=None):
    config, rc = get_region_conf_rc(name=name)
    neutron = neutron_connect(rc)
    region_objects = []
    request = 'neutron.list_' + objects + '()[\'' + objects + '\']'
    for obj in eval(request):
        if name in obj[u'name']:
            region_objects.append(obj)
    return region_objects


def neutron_update_ports(name=None):
    config, rc = get_region_conf_rc(name=name)
    neutron = neutron_connect(rc)
    net_list = neutron_get_objects(name=name, objects='networks')

# we don't utilize neutron_get_objects, cause we need all tenant ports
    for port in neutron.list_ports()['ports']:
        for net in net_list:
            if port[u'network_id'] == net['id'] and\
               port[u'name'] == '':
                port_update = {'port': {u'name': net[u'name'] + '_port'}}
                neutron.update_port(port[u'id'], port_update)


def neutron_purge(prefix=None, rc=None):
#   we connect to neutron to retrieve the list of networks
    neutron = neutron_connect(rc)
    net_list = neutron.list_networks()['networks']

#   we seek and destroy any region related network
#   retrieving data from neutron network list for this region
    region_net_list = []
    for network in net_list:
        if prefix in network[u'name']:
            region_net_list.append(network)

#   now we get port list, and we will test if each port relate to any
#   network for this region
    for port in neutron.list_ports()['ports']:
        for network in region_net_list:
            # if it matches, we delete it
            if port[u'network_id'] == network[u'id']:
                # we can't delete if a router is owning this port
                if port['device_owner'] == 'network:router_interface':
                    port_rm = {'port_id': port[u'id']}
                    neutron.remove_interface_router(port[u'device_id'],
                                                    port_rm)
                else:
                    neutron.delete_port(port[u'id'])

#   we can delete networks now
    for network in region_net_list:
        print('deleting network:' +
              'name:"{}", id:"{}"'.format(network[u'name'],
                                          network[u'id']))
        neutron.delete_network(network[u'id'])

    router_list = neutron.list_routers()['routers']
    for router in router_list:
        if prefix in router[u'name']:
            print('deleting router:' +
                  'name:"{}", id:"{}"'.format(router[u'name'],
                                              router[u'id']))
            neutron.delete_router(router[u'id'])


def main_check(args):
    # Opening of files
    log = open(LOG_FILE, "w")
    log_null = open(os.devnull, "w")
    log.write('#####################################################\n')

    # initialisation of command to be executed later
    pkg_inst = ['sudo', 'apt-get', 'install']

    # initialisation of package to install (if any)
    check_list = ''
    state_package = 'present'

    # Test if install with --yes/-y options
    if args.yes:
        pkg_inst.append('-y')

    # what to do if we need to upgrade
    if args.update:
        log.write('updating package-list\n')
        # We start the update of package list
        retval = subprocess.call(['sudo', 'apt-get', 'update'])
        if retval != 0:
            log.write('update failed!')
            print('check apt config, look at log file: "{}"'.format(LOG_FILE))

    # We check if any of the needed package are missing
    for check in sorted(DEPENDANCIES.keys()):

        cmd_line = 'dpkg -s ' + check + \
            ' | grep Status | grep installed |wc -l'
        output = subprocess.check_output(cmd_line, shell=True)

        # if missing, we add it to the install command and to the list
        if output == 0 or not os.path.isfile(which(DEPENDANCIES[check])):
            print("Package \"{}\" is not installed,".format(check) +
                  " it is added to install list.")
            pkg_inst.append(str(check))
            check_list = check_list + ' ' + str(check)
            print('Check: \"{}\", is not installed'.format(check))
        else:
            print('Check: \"{}\", is {}'.format(check, state_package))

    # we display and log only if we need to install some packages
    if check_list != '':
        print('\nWe need to install: \"{}\"\n'.format(check_list))
        log.write('need to install the following packages:\n' +
                  '{}\n'.format(check_list))
        # real installation command is issued:
#        retval = subprocess.call(pkg_inst,stdout=log,stderr=subprocess.STDOUT)
        retval = subprocess.call(pkg_inst)

        # package have been installed, they were not all present
        state_package = 'installed'

        if retval != 0:
            print('A problem occurred, check log')
            log.write('A problem occurred!\n')
            log.write(
                '#####################################################\n')
            sys.exit(2)

    print('All needed package are {}, go on...'.format(state_package))
    log.write('All packages {}! Go on...\n'.format(state_package))

    log.write('#####################################################\n')
    log.close()
    log_null.close()


def main_region_switch(args):
    if not os.path.exists(DEFAULT_CONF_DIR + '/' + args.region_name + '/' +
                          args.region_name + '.conf'):
        print('please provide an existing region, you can create it ' +
              'with region-create command')
        sys.exit()
    set_modev_region(args.region_name)


def main_region_create(args):
    region_name = select_region(region=args.region_name)

#   create directory if necessary
    region_dir = DEFAULT_CONF_DIR + '/' + region_name
    if not os.path.exists(region_dir):
        os.makedirs(region_dir)

#   name of the configuration file
    region_file = region_dir + '/' + region_name + '.conf'

#   we test if region isn't already configured
    if os.path.exists(region_file) and args.force is False:
        print("this region is already configured!")
        sys.exit()

#   we add a '_' at the end of the region if it doesn't exist
    region = region_name
    if region[-1] is not '_':
        region = region + '_'
    full_region = FIXED_PREFIX + region
    # we start building configuration
    openrc_cfg = {}
    region_cfg = {}

#   we read data from openrc file
    if not os.path.isfile(args.openrc):
        print('openrc file: "{}" don\'t exist!'.format(args.openrc))
        sys.exit()

    openrc_cfg.update({k: '' for k in CFG_NAMES})
    found = load_openrc(openrc=args.openrc,)
    openrc_cfg.update(found)

    # we test if this region is unconfigured but exists:
    for existing_region in get_openrc_regions(openrc_cfg):
        if existing_region == region_name:
            print('this region is unconfigured, but seems to exists!' +
                  ' Try a rebuild process (not implemented yet)')
            sys.exit(2)

#    logging.basicConfig(level=logging.DEBUG)
    neutron = neutron_connect(openrc_cfg)

    extern_id = u''
#   choosing external network:
    if args.net_extern is None:
        for network in neutron.list_networks()['networks']:
            if network['router:external'] is True:
                print('external network chosen:' +
                      ' "{}"'.format(network['name']))
                extern_id = network['id']
                break
            else:
                print('There is no external network here, quit')
                sys.exit(2)
    else:
        extern_id = neutron.list_networks(id=args.net_extern)
        if len(extern_id['networks']) is 0:
            print('This external network id is unknown:' +
                  '"{}"'.format(args.net_extern))
            sys.exit()

        elif extern_id['networks'][0]['router:external'] is False:
            print('This network is not a external network' +
                  ': "{}"'.format(args.net_extern))
            sys.exit()
        else:
            extern_id = args.net_extern
    region_cfg.update({'external_network': extern_id})

#   creating a new router or using an existing one
    if args.net_router is None:
    #   creating a new router:
        router = {'name': FIXED_PREFIX + region + 'router', }
        router = neutron.create_router({'router': router})
        router = router['router']
        print(u'router created, \n\tname:{} '.format(router[u'name']) +
              '\n\tid:{}'.format(router['id']))
        router_id = router['id']
    #   linking router to external network
        neutron.add_gateway_router(router_id, {"network_id": extern_id})

#   checking and using the one provided:
    else:
        # using a former one, we chek if it exists:
        router = neutron.list_routers(id=args.net_router)

        if len(router['routers']) is 0:
            print('This router Id doesn\'t exist!')
            sys.exit()
        else:
            router_id = router['routers'][0]['id']
            print(router_id)
    region_cfg.update({'router-id': router_id})

#   creating network admin:
    network = {'name': full_region + 'admin',
               'admin_state_up': True}
    network = neutron.create_network({'network': network})
    network_id = network[u'network'][u'id']
    region_cfg.update({'networks': [{'name': 'net-admin-id',
                                    'network-id': network_id}]})

#   creating network admin subnet:
    cidr = '192.168.0.0/24'
    subnet = {'name': full_region + 'adm-snet',
              'network_id': network_id, 'ip_version': 4,
              'cidr': cidr}
    subnet = neutron.create_subnet({'subnet': subnet})
    subnet_id = subnet[u'subnet'][u'id']
    region_cfg.update({'subnets': [{'name': 'subnet-adm-id',
                                   'subnet-id': subnet_id,
                                   'subnet-net': cidr}]})

#   connecting network admin to router
    port = {'name': full_region + 'ext-adm', 'network_id': network_id}
    port = neutron.create_port({'port': port})
    port_id = port[u'port'][u'id']
    neutron.add_interface_router(router_id,
                                 {'port_id': port_id})

#   preparing to serialize
    region_conf = {'region-name': region_name,
                   'openrc': openrc_cfg,
                   'config': region_cfg, }

#   creating conf file
    fp = open(region_file, "w")

    fp.write(yaml.safe_dump(region_conf, default_flow_style=False))

    fp.close()

    # updating ports names:
    neutron_update_ports(name=region_name)

    print('switching to region: "{}"'.format(region_name))
    set_modev_region(region_name)


def main_region_current(args):
    region = get_modev_region()
    if is_modev_region(region):
        state = 'Config file exists, state OK'
    else:
        state = 'Config file is destroyed, state KO'
    print('Current selected region:\n{}, {}'.format(region, state))


def main_region_list(args):
    default_region = get_modev_region()
    for region in list_modev_regions():
        if region == default_region:
            print('*\t' + region)
        else:
            print('\t' + region)
    sys.exit()


def get_openrc_regions(rc=None):
    net_list = []
    neutron = neutron_connect(rc)

    for name in neutron.list_networks()['networks']:
        if 'modev' in name[u'name']:
        #   we retrieve the name of the region from network name
            net_list.append(name[u'name'].split('_')[1])
    return(net_list)


def purge_openrc(rc=None):
    for region in get_openrc_regions(rc):
        neutron_purge(FIXED_PREFIX + region, rc)
        rmdir(DEFAULT_CONF_DIR + region)


def purge_region(region=None):
    if region is None:
        raise NameError('Error, region can\'t be None, please report')
    region_rc = get_region_rc(region)
    neutron_purge(FIXED_PREFIX + region, region_rc)
    rmdir(DEFAULT_CONF_DIR + region)


def main_region_purge(args):
    region = select_region(args.region_name)
    if args.all_regions:
        for region in list_modev_regions():
            region_rc = get_region_rc(region)
            purge_openrc(region_rc)
    elif args.openrc:
        region_rc = load_openrc(openrc=args.openrc)
        purge_openrc(region_rc)
    elif region:
        region = select_region(args.region_name)
        if args.same_cloud:
            region_rc = get_region_rc(region)
            purge_openrc(region_rc)
        else:
            purge_region(region)
    else:
        print('You must specifies either : \nan openrc file (-rc)\n' +
              'a region name (-rn)\n--all-regions to remove anything')
        sys.exit(2)


def main_net_extern_list(args):
    region_ext = ''

    if args.region_name is None and args.openrc is None:
        print('Warning, I need either an openrc file or a' +
              ' region name to get data from ! (-rc -rn)')
        sys.exit()

    if args.region_name is not None:
        region_prefix = FIXED_PREFIX + args.region_name
        # we get region config
        region_conf = load_region_file(args.region_name)
        # we retrieve open_rc part
        region_conf = region_conf[region_prefix]
        region_ext = region_conf[1]['config']['external_network']
        region_rc = region_conf[0]['openrc']

    else:
        region_rc = load_openrc(openrc=args.openrc)

    # we connect to neutron to retrieve the list of networks
    neutron = neutron_connect(region_rc)
    net_list = neutron.list_networks()
    net_list = net_list['networks']
    for network in net_list:
        if network['router:external'] is True:
            if region_ext == network['id']:
                default = '*'
            else:
                default = ' '
            print(default + 'name: "{}"'.format(network['name']) +
                  ', id: "{}"'.format(network['id']))

'''
def main_net_extern(args):
    raise NotImplementedError()


def main_cluster_add(args):
    raise NotImplementedError()


def main_cluster_delete(args):
    raise NotImplementedError()


def main_cluster_list(args):
    raise NotImplementedError()


def main_cluster_node_list(args):
    raise NotImplementedError()


def main_cluster_node_add(args):
    raise NotImplementedError()


def main_cluster_node_delete(args):
    raise NotImplementedError()


def main_node_list(args):
    raise NotImplementedError()


def main_node_add(args):
    raise NotImplementedError()


def main_node_delete(args):
    raise NotImplementedError()


def main_node_console(args):
    raise NotImplementedError()


def main_node_cluster_change(args):
    raise NotImplementedError()


def main_node_profile_list(args):
    raise NotImplementedError()


def main_node_profile_add(args):
    raise NotImplementedError()


def main_node_profile_delete(args):
    raise NotImplementedError()


def main_node_role_list(args):
    raise NotImplementedError()


def main_node_role_add(args):
    raise NotImplementedError()


def main_node_role_del(args):
    raise NotImplementedError()


def main_maas_region_controller_list(args):
    raise NotImplementedError()


def main_maas_region_controller_add(args):
    raise NotImplementedError()


def main_maas_region_controller_del(args):
    raise NotImplementedError()


def main_maas_cluster_controller_list(args):
    raise NotImplementedError()


def main_maas_cluster_controller_add(args):
    raise NotImplementedError()


def main_maas_cluster_controller_del(args):
    raise NotImplementedError()


def main_destroy(args):
    raise NotImplementedError()


def main_maas_install(args):
    raise NotImplementedError()


def main_maas_get_creds(args):
    raise NotImplementedError()


def main_maas_create_user(args):
    raise NotImplementedError()


def main_node_new(args):
    raise NotImplementedError()


def main_node_list(args):
    raise NotImplementedError()


def main_node_delete(args):
    raise NotImplementedError()


def main_node_console(args):
    raise NotImplementedError()


def main_node_delete_all(args):
    raise NotImplementedError()


def main_network_setup(args):
    raise NotImplementedError()


def main_maas_get_creds(args):
    raise NotImplementedError()


def main_maas_create_user(args):
    user = args.user
    passwd = args.password
    if passwd is None:
        passwd = user

    cmd = ['sudo', 'maas', 'createadmin', '--username=%s' % args.user,
           '--password=%s' % passwd, "--email=%s@modev.example.com"]

    try:
        (out, err) = util.subp(cmd, capture=True)
        return 0
    except util.CalledProcessError as e:
        sys.stderr.write(e + "\n")
        sys.stderr.write("Failed to create user")
        return 1
'''

'''
def main_region_enter(args):
    raise NotImplementedError()


def main_region_backup(args):
    raise NotImplementedError()


def main_region_restore(args):
    raise NotImplementedError()


def main_region_import(args):
    raise NotImplementedError()


def main_region_export(args):
    raise NotImplementedError()

'''

'''
def main_net_base_conf(args):
    raise NotImplementedError()


def main_maas_all_in_one(args):
    raise NotImplementedError()


def main_maas_with_clusters(args):
    raise NotImplementedError()


def main_net_base_state(args):
    raise NotImplementedError()

'''


def main_td(args):
    neutron_update_ports(args.rn)


def main():
    parser = argparse.ArgumentParser()

    # Top level args
    for (args, kwargs) in COMMON_ARGS:
        parser.add_argument(*args, **kwargs)

    subparsers = parser.add_subparsers()

    for subcmd in sorted(SUBCOMMANDS.keys()):
        val = SUBCOMMANDS[subcmd]
        sparser = subparsers.add_parser(subcmd, help=val['help'])
        mfuncname = 'main_' + subcmd.replace('-', '_')
        sparser.set_defaults(action=globals()[mfuncname])
        if 'opts' in val.keys():
            for (args, kwargs) in val['opts']:
                if isinstance(args, str):
                    args = [args]
                # we search if we have any named parameter with '-' in it
                # and if yes we replace it by '_' (ie: region-name)
                # we transtype tuple args into a string named temp
                temp = ''.join(args)
                # if temp has a '-' in it and it is not the '--'
                # from positional parameters
                if temp.find('-') > 1:
                    # we replace '-' by '_'
                    temp = temp.replace('-', '_')
                    # we transtype back to tuple for parser to be happy
                    args = tuple([temp])
                sparser.add_argument(*args, **kwargs)

    args = parser.parse_args()
    if not getattr(args, 'action', None):
        # http://bugs.python.org/issue16308
        parser.print_help()
        return 1
    return args.action(args)


if __name__ == '__main__':
    import sys
    sys.exit(main())

# vi: ts=4 expandtab syntax=python
